==> org.kikaineko.mock.plugin.kikainekoMocker_1.0.1.zip: org/kikaineko/mock/framework/DestroyMethodSeakerFilter.java <==
package org.kikaineko.mock.framework;

import java.util.Hashtable;
import java.util.Vector;

import org.kikaineko.util.IntArray;
import org.kikaineko.util.StringArray;

public class DestroyMethodSeakerFilter implements Filter {
	StringArray destroyMethodNames = new StringArray();

	public TargetClass execute(TargetClass tc) {
		tc= seaking(tc);
		qXg[^[ɔj󂩂ǂZbg(tc);
		return tc;
	}
	
	private void qXg[^[ɔj󂩂ǂZbg(TargetClass tc) {
		for(int symbol=0;symbol<TargetHistoryTerm.count();symbol++){
			TargetHistoryTerm tht=TargetHistoryTerm.getBySymbol(symbol);
			if(!tht.isDestroy()){
				if(destroyMethodNames.contains(tht.getMethodName())){
					tht.mustBesDestroy();
				}
			}
		}
	}

	protected TargetClass seaking(TargetClass tc) {
		for (int i = 0; i < tc.howManyMethods(); i++) {
			if (tc.getReturnType(i).equals("void")) {
				destroyMethodNames.add(tc.getMethodName(i));
			}
		}
		IntArray[] ΏۃO[v = Ō̃\bhĂяolقȂhitorỹV{̏WԂ(tc);

		for (int i = 0; i < ΏۃO[v.length; i++) {
			determinDestroy(ΏۃO[v[i]);
		}
		for (int i = 0; i < tc.howManyMethods(); i++) {
			if (destroyMethodNames.contains(tc.getMethodName(i))) {
				tc.mustBeDestoryMethodThis(i);
			}
		}
		return tc;
	}

	private void determinDestroy(IntArray ΏۃqXg[s) {
		IntArray historySymbols = ܂łȂqXgV{Q(ΏۃqXg[s,
				destroyMethodNames);
		if (historySymbols.length() == 0) {
			return;
		}
		IntArray[] histories = qXgV{Q^[̔zɕϊ(historySymbols);
		histories = DestroyMethodSeakerHelper.Ō̌Ăяo͋ʂ̂Ȃ̂ō(histories);
		histories = DestroyMethodSeakerHelper.Őʍ(histories);

		Hashtable cash = LbV(histories, historySymbols);
		histories = LbVqXg^[̔z擾(historySymbols, cash);
		
		while (historySymbols.length() != 0) {
			܂j󃁃\bhɎw肳ĂȂ̂1w肷(histories, destroyMethodNames);
			historySymbols = ܂łȂqXgV{Q(ΏۃqXg[s, destroyMethodNames);
			histories = LbVqXg^[̔z擾(historySymbols, cash);
		}
	}

	private Hashtable LbV(IntArray[] histories, IntArray historySymbols) {
		Hashtable cash = new Hashtable();
		for (int i = 0; i < histories.length; i++) {
			if (histories[i].length() != 0) {
				cash.put(new Integer(historySymbols.get(i)), histories[i]);
			}
		}
		return cash;
	}

	private IntArray[] LbVqXg^[̔z擾(IntArray historySymbols,
			Hashtable cash) {
		int leng = 0;
		for (int i = 0; i < historySymbols.length(); i++) {
			if (cash.containsKey(new Integer(historySymbols.get(i)))) {
				leng++;
			}
		}
		if (leng == 0) {
			return new IntArray[0];
		}
		IntArray[] histories = new IntArray[leng];
		int index = 0;
		for (int i = 0; i < historySymbols.length(); i++) {

			IntArray temp = (IntArray) cash.get(new Integer(historySymbols
					.get(i)));
			if (temp != null) {
				histories[index++] = temp.createCopy();
			}
		}
		return histories;
	}

	protected IntArray[] qXgV{Q^[̔zɕϊ(IntArray historySymbols) {
		IntArray[] histories = new IntArray[historySymbols.length()];
		for (int j = 0; j < histories.length; j++) {
			histories[j] = TargetHistory.getTargetHistoryBySymbol(
					historySymbols.get(j)).toIntArray();
		}
		return histories;
	}

	/**
	 * ƂĂĂȂASYEEE
	 * 
	 * @param historySymbols
	 * @param destroyMethodNames
	 */
	protected void ܂j󃁃\bhɎw肳ĂȂ̂1w肷(IntArray[] historySymbols,
			StringArray destroyMethodNames) {
		for (int index = 0; index < DestroyMethodSeakerHelper
				.łqXg[̒(historySymbols); index++) {
			for (int i = 0; i < historySymbols.length; i++) {
				IntArray history = historySymbols[i];
				if (index < history.length()) {
					TargetHistoryTerm tht = TargetHistoryTerm
							.getBySymbol(history.get(index));
					if (!destroyMethodNames.contains(tht.getMethodName())) {
						destroyMethodNames.add(tht.getMethodName());
						return;
					}
				}
			}
		}

	}

	protected IntArray[] Ō̃\bhĂяolقȂhitorỹV{̏WԂ(TargetClass tc) {
		StringArray[] datas = new StringArray[TargetHistoryTerm.count()];
		IntArray[] tempIa = new IntArray[TargetHistoryTerm.count()];
		for (int i = 0; i < tc.howManyHistories(); i++) {
			TargetHistory history = tc.getHistoryAt(i);
			int symbol = history.lastCallingTermAsSymbol();
			if (datas[symbol] == null) {
				datas[symbol] = new StringArray(5);
				tempIa[symbol] = new IntArray(5);
			}
			datas[symbol].add(tc.getValueAtHistory(i));
			tempIa[symbol].add(history.getHistorySymbol());
		}

		Vector vec = new Vector();
		for (int symbol = 0; symbol < datas.length; symbol++) {
			if (datas[symbol] != null && datas[symbol].length() > 1) {
				if (!datas[symbol].isUniq()) {
					vec.add(tempIa[symbol]);
				}
			}
		}

		tempIa = new IntArray[vec.size()];
		for (int i = 0; i < tempIa.length; i++) {
			tempIa[i] = (IntArray) vec.get(i);
		}
		return tempIa;
	}

	protected IntArray ܂łȂqXgV{Q(IntArray historySymbols,
			StringArray destroyMethodNames) {
		StringArray la = new StringArray();
		for (int i = 0; i < historySymbols.length(); i++) {
			TargetHistory th = TargetHistory
					.getTargetHistoryBySymbol(historySymbols.get(i));
			la.add(toIntArray(th, destroyMethodNames, (TargetHistoryTerm
					.count() + 1)));
		}
		IntArray indexs = łȂindexs(la);
		IntArray results = new IntArray();
		for (int i = 0; i < indexs.length(); i++) {
			results.add(historySymbols.get(i));
		}
		return results;
	}

	private IntArray łȂindexs(StringArray la) {
		return la.dlindexs();
	}

	private String toIntArray(TargetHistory th, StringArray destroyMethodNames,
			int max) {
		StringBuffer hash=new StringBuffer();
		for (int i = 0; i < th.length(); i++) {
			TargetHistoryTerm tht = th.getTargetHistoryTermAt(i);
			if (destroyMethodNames.contains(tht.getMethodName())) {
				hash.append(tht.getSymbol());
				hash.append(":");
			}
		}
		return hash.toString();
	}
}

==> org.kikaineko.mock.plugin.kikainekoMocker_1.1a.zip: org/kikaineko/mock/framework/DestroyMethodSeakerFilter.java <==
package org.kikaineko.mock.framework;

import java.util.Hashtable;
import java.util.Vector;

import org.kikaineko.util.IntArray;
import org.kikaineko.util.StringArray;

public class DestroyMethodSeakerFilter implements Filter {
	StringArray destroyMethodNames = new StringArray();

	public TargetClass execute(TargetClass tc) {
		tc = seaking(tc);
		qXg[^[ɔj󂩂ǂZbg(tc);
		return tc;
	}

	private void qXg[^[ɔj󂩂ǂZbg(TargetClass tc) {
		for (int symbol = 0; symbol < TargetHistoryTerm.count(); symbol++) {
			TargetHistoryTerm tht = TargetHistoryTerm.getBySymbol(symbol);
			if (!tht.isDestroy()) {
				if (destroyMethodNames.contains(tht.getMethodName())) {
					tht.mustBesDestroy();
				}
			}
		}
	}

	protected TargetClass seaking(TargetClass tc) {
		for (int i = 0; i < tc.howManyMethods(); i++) {
			if (tc.getReturnType(i).equals("void")) {
				destroyMethodNames.add(tc.getMethodName(i));
			}
		}
		IntArray[] ΏۃO[v = Ō̃\bhĂяolقȂhitorỹV{̏WԂ(tc);
		
		for (int i = 0; i < ΏۃO[v.length; i++) {
			determinDestroy(ΏۃO[v[i]);
		}
		for (int i = 0; i < tc.howManyMethods(); i++) {
			if (destroyMethodNames.contains(tc.getMethodName(i))) {
				tc.mustBeDestoryMethodThis(i);
			}
		}
		return tc;
	}

	private void determinDestroy(IntArray ΏۃqXg[s) {
		IntArray historySymbols = ܂łȂqXgV{Q(ΏۃqXg[s,
				destroyMethodNames);
		if (historySymbols.length() == 0) {
			return;
		}
		IntArray[] histories = qXgV{Q^[̔zɕϊ(historySymbols);
		histories = DestroyMethodSeakerHelper.Ō̌Ăяo͋ʂ̂Ȃ̂ō(histories);

		histories = DestroyMethodSeakerHelper.Őʍ(histories);
		Hashtable cash = LbV(histories, historySymbols);
		histories = LbVqXg^[̔z擾(historySymbols, cash);

		while (historySymbols.length() != 0) {
			boolean flag = ܂j󃁃\bhɎw肳ĂȂ̂1w肷(histories,
					destroyMethodNames);
			if (flag) {
				historySymbols = ܂łȂqXgV{Q(ΏۃqXg[s, destroyMethodNames);
				histories = LbVqXg^[̔z擾(historySymbols, cash);
			} else {
				return;
			}
		}
	}

	private Hashtable LbV(IntArray[] histories, IntArray historySymbols) {
		Hashtable cash = new Hashtable();
		for (int i = 0; i < histories.length; i++) {
			if (histories[i].length() != 0) {
				cash.put(new Integer(historySymbols.get(i)), histories[i]);
			}
		}
		return cash;
	}

	private IntArray[] LbVqXg^[̔z擾(IntArray historySymbols,
			Hashtable cash) {
		int leng = 0;
		for (int i = 0; i < historySymbols.length(); i++) {
			if (cash.containsKey(new Integer(historySymbols.get(i)))) {
				leng++;
			}
		}
		if (leng == 0) {
			return new IntArray[0];
		}
		IntArray[] histories = new IntArray[leng];
		int index = 0;
		for (int i = 0; i < historySymbols.length(); i++) {

			IntArray temp = (IntArray) cash.get(new Integer(historySymbols
					.get(i)));
			if (temp != null) {
				histories[index++] = temp.createCopy();
			}
		}
		return histories;
	}

	protected IntArray[] qXgV{Q^[̔zɕϊ(IntArray historySymbols) {
		IntArray[] histories = new IntArray[historySymbols.length()];
		for (int j = 0; j < histories.length; j++) {
			histories[j] = TargetHistory.getTargetHistoryBySymbol(
					historySymbols.get(j)).toIntArray();
		}
		return histories;
	}

	/**
	 * ƂĂĂȂASYEEE
	 * 
	 * @param historySymbols
	 * @param destroyMethodNames
	 */
	protected boolean ܂j󃁃\bhɎw肳ĂȂ̂1w肷(IntArray[] historySymbols,
			StringArray destroyMethodNames) {
		for (int index = 0; index < DestroyMethodSeakerHelper
				.łqXg[̒(historySymbols); index++) {
			for (int i = 0; i < historySymbols.length; i++) {
				IntArray history = historySymbols[i];
				
				if (index < history.length()) {
					TargetHistoryTerm tht = TargetHistoryTerm
							.getBySymbol(history.get(index));
					if (!destroyMethodNames.contains(tht.getMethodName())) {
						destroyMethodNames.add(tht.getMethodName());
						return true;
					}
				}
			}
		}
		return false;
	}

	protected IntArray[] Ō̃\bhĂяolقȂhitorỹV{̏WԂ(TargetClass tc) {
		StringArray[] datas = new StringArray[TargetHistoryTerm.count()];
		IntArray[] tempIa = new IntArray[TargetHistoryTerm.count()];
		for (int i = 0; i < tc.howManyHistories(); i++) {
			TargetHistory history = tc.getHistoryAt(i);
			int symbol = history.lastCallingTermAsSymbol();
			if (datas[symbol] == null) {
				datas[symbol] = new StringArray(5);
				tempIa[symbol] = new IntArray(5);
			}
			datas[symbol].add(tc.getValueAtHistory(i));
			tempIa[symbol].add(history.getHistorySymbol());
		}

		Vector vec = new Vector();
		for (int symbol = 0; symbol < datas.length; symbol++) {
			if (datas[symbol] != null && datas[symbol].length() > 1) {
				if (!datas[symbol].isUniq()) {
					vec.add(tempIa[symbol]);
				}
			}
		}

		tempIa = new IntArray[vec.size()];
		for (int i = 0; i < tempIa.length; i++) {
			tempIa[i] = (IntArray) vec.get(i);
		}
		return tempIa;
	}

	protected IntArray ܂łȂqXgV{Q(IntArray historySymbols,
			StringArray destroyMethodNames) {
		StringArray la = new StringArray();
		for (int i = 0; i < historySymbols.length(); i++) {
			TargetHistory th = TargetHistory
					.getTargetHistoryBySymbol(historySymbols.get(i));
			la.add(toIntArray(th, destroyMethodNames));
		}
		IntArray indexs = łȂindexs(la);
		IntArray results = new IntArray();
		for (int i = 0; i < indexs.length(); i++) {
			results.add(historySymbols.get(indexs.get(i)));
		}
		return results;
	}

	private IntArray łȂindexs(StringArray la) {
		return la.dlindexs();
	}

	private String toIntArray(TargetHistory th, StringArray destroyMethodNames) {
		StringBuffer hash = new StringBuffer();
		for (int i = 0; i < th.length(); i++) {
			TargetHistoryTerm tht = th.getTargetHistoryTermAt(i);
			if (destroyMethodNames.contains(tht.getMethodName())) {
				hash.append(tht.getSymbol());
				hash.append(":");
			}
		}
		return hash.toString();
	}
}
